<?php

namespace InSegment\ApiCore\Models;

use InSegment\ApiCore\Services\Transactor;
use InSegment\ApiCore\Services\BufferedBuilder;

use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Eloquent\Model;

abstract class SupplementaryTable extends Model
{
    /**
     * Constructor
     * 
     * @param array $attributes
     */
    public function __construct(array $attributes = array(), $uid = null)
    {
        parent::__construct($attributes);
        
        $database = Transactor::getTransactionSchema();
        $table = substr(implode('_', [$this->getTable(), $uid ?? Transactor::getTransactionUID()]), 0, 64);
        $this->table = "{$database}.{$table}";
    }
    
    /**
     * Indicate there is no incrementing key
     * 
     * @var bool 
     */
    public $incrementing = false;

    /**
     * The model has timestamps
     * 
     * @var bool
     */
    public $timestamps = false;
    
    /**
     * Create a new Eloquent query builder for the model.
     *
     * @param  \Illuminate\Database\Query\Builder  $query
     * @return \InSegment\ApiCore\Services\BufferedBuilder|static
     */
    public function newEloquentBuilder($query)
    {
        return new BufferedBuilder($query);
    }
    
    /**
     * Run the migrations.
     *
     * @param array $options
     * @param bool $inMemory
     * @return void
     */
    public abstract static function establish(array $options, bool $inMemory);

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public static function disband()
    {
        Schema::dropIfExists((new static)->getTable());
    }
}
