<?php

namespace InSegment\ApiCore\Models;

use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;

class UUIDGeneration extends SupplementaryTable
{
    /**
     * Run the migrations.
     *
     * @param array $enabledTables
     * @param bool $inMemory
     * @return void
     */
    public static function establish(array $enabledTables, bool $inMemory)
    {
        Schema::create((new static)->getTable(), function (Blueprint $table) use ($enabledTables, $inMemory) {
            if ($inMemory) {
                $table->engine = 'MEMORY';
            }
            
            $table->unsignedInteger('id')->nullable();
            $table->unsignedBigInteger('uuid');
            $table->enum('table', $enabledTables);
            $table->primary('uuid');
            $table->index('table', 'uuid_generation_table_index');
        });
    }
    
    /**
     * Get statement generating amount if ids for table and key
     * 
     * @param string $table
     * @param int $rows
     * @return string
     */
    public static function getGeneratorStatement(string $table, int $rows)
    {
        list($schema, $myTable) = explode('.', (new static)->getTable());
        $nRows = implode(', ', array_fill(0, $rows, '()'));
        
        return "CREATE TRIGGER `{$schema}`.`trigger_{$myTable}` BEFORE INSERT ON `{$schema}`.`{$myTable}`"
            . " FOR EACH ROW SET NEW.`uuid` = UUID_SHORT(), NEW.`table` = '{$table}';"
            . "INSERT INTO `{$schema}`.`{$myTable}` () VALUES {$nRows};"
            . "DROP TRIGGER `{$schema}`.`trigger_{$myTable}`;";
    }
}
